﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Compound bodies are created from other entities to make concave shapes.
    /// </summary>
    public class CompoundBodiesDemo : StandardDemo
    {
        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public CompoundBodiesDemo(DemosGame game)
            : base(game)
        {
            //Build the first body
            var cb1 = new CompoundBody();

            var a = new Sphere(new Vector3(0, 1, 0), .5f, 15);
            var b = new Cone(new Vector3(1, 1, 0), 2f, .5f, 15);
            var c = new Sphere(new Vector3(-1, 1, 0), .5f, 15);
            cb1.AddBody(a);
            cb1.AddBody(b);
            cb1.AddBody(c);


            //Build the second body
            var cb2 = new CompoundBody();
            var d = new Box(new Vector3(0, 3, 0), 1, 1, 1, 2);
            var f = new Box(new Vector3(1, 3.5f, 0), 1, 1, 1, 2);
            cb2.AddBody(d);
            cb2.AddBody(f);

            //Build the third Braum's-fry style body
            var cb3 = new CompoundBody();
            for (int k = 0; k < 7; k++)
            {
                cb3.AddBody(new Box(new Vector3(-4 + k * .7f, 2 + .7f * k, 2 + k * .2f), 1, 1, 1, 1));
            }
            //Add them all to the space
            Space.Add(cb3);
            Space.Add(cb2);
            Space.Add(cb1);


            Space.Add(new Box(new Vector3(0, -.5f, 0), 10, 1, 10));
            game.Camera.Position = new Vector3(0, 3, 15);
        }

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string Name
        {
            get { return "Compound Bodies"; }
        }
    }
}