﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Constraints;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using BEPUphysics.MathExtensions;

namespace BEPUphysicsDemos
{
    /// <summary>
    /// Graphical representation of a LinearLimit.
    /// </summary>
    public class DisplayLinearLimit: LineDisplayObject
    {

        Line aToConnection;
        Line bToConnection;
        Line error;

        Line edgeAB, edgeBC, edgeCD, edgeDA; //Top edges.
        Line edgeEF, edgeFG, edgeGH, edgeHE; //Bottom edges.
        Line edgeAE, edgeBF, edgeCG, edgeDH; //Side edges.

        public DisplayLinearLimit(LinearLimit constraint, LineDrawer drawer)
            : base(drawer, constraint)
        {

            aToConnection = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            bToConnection = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            error = new Line(Color.Red, Color.Red, drawer);

            //Corners
            edgeAB = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeBC = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeCD = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeDA = new Line(Color.DarkBlue, Color.DarkBlue, drawer);

            edgeEF = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeFG = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeGH = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeHE = new Line(Color.DarkBlue, Color.DarkBlue, drawer);

            edgeAE = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeBF = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeCG = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            edgeDH = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
        }


        /// <summary>
        /// Moves the constraint lines to the proper location relative to the entities involved.
        /// </summary>
        public override void update()
        {

            //Move lines around
            LinearLimit constraint = this.lineObject as LinearLimit;

            //Connection and error
            aToConnection.positionA = constraint.connectionA.centerOfMass;
            aToConnection.positionB = constraint.anchorA;

            bToConnection.positionA = constraint.connectionB.centerOfMass;
            bToConnection.positionB = constraint.anchorB;

            error.positionA = aToConnection.positionB;
            error.positionB = bToConnection.positionB;

            Matrix transform = Matrix3x3.ToMatrix4x4(constraint.basis.worldTransform);
            Vector3 a = constraint.anchorA + Vector3.TransformNormal(constraint.maximum, transform);
            Vector3 b = constraint.anchorA + Vector3.TransformNormal(new Vector3(constraint.maximum.X, constraint.maximum.Y, constraint.minimum.Z), transform);
            Vector3 c = constraint.anchorA + Vector3.TransformNormal(new Vector3(constraint.minimum.X, constraint.maximum.Y, constraint.minimum.Z), transform);
            Vector3 d = constraint.anchorA + Vector3.TransformNormal(new Vector3(constraint.minimum.X, constraint.maximum.Y, constraint.maximum.Z), transform);
            Vector3 e = constraint.anchorA + Vector3.TransformNormal(new Vector3(constraint.maximum.X, constraint.minimum.Y, constraint.maximum.Z), transform);
            Vector3 f = constraint.anchorA + Vector3.TransformNormal(new Vector3(constraint.maximum.X, constraint.minimum.Y, constraint.minimum.Z), transform);
            Vector3 g = constraint.anchorA + Vector3.TransformNormal(constraint.minimum, transform);
            Vector3 h = constraint.anchorA + Vector3.TransformNormal(new Vector3(constraint.minimum.X, constraint.minimum.Y, constraint.maximum.Z), transform);

            edgeAB.positionA = a;
            edgeAB.positionB = b;
            edgeBC.positionA = b;
            edgeBC.positionB = c;
            edgeCD.positionA = c;
            edgeCD.positionB = d;
            edgeDA.positionA = d;
            edgeDA.positionB = a;

            edgeAE.positionA = a;
            edgeAE.positionB = e;
            edgeBF.positionA = b;
            edgeBF.positionB = f;
            edgeCG.positionA = c;
            edgeCG.positionB = g;
            edgeDH.positionA = d;
            edgeDH.positionB = h;

            edgeEF.positionA = e;
            edgeEF.positionB = f;
            edgeFG.positionA = f;
            edgeFG.positionB = g;
            edgeGH.positionA = g;
            edgeGH.positionB = h;
            edgeHE.positionA = h;
            edgeHE.positionB = e;
        }

        /// <summary>
        /// Removes the lines used by the display constraint from its line drawer.
        /// </summary>
        public override void removeLines()
        {
            drawer.removeLine(aToConnection);
            drawer.removeLine(bToConnection);
            drawer.removeLine(error);
            drawer.removeLine(edgeAB);
            drawer.removeLine(edgeBC);
            drawer.removeLine(edgeCD);
            drawer.removeLine(edgeDA);

            drawer.removeLine(edgeEF);
            drawer.removeLine(edgeFG);
            drawer.removeLine(edgeGH);
            drawer.removeLine(edgeHE);

            drawer.removeLine(edgeAE);
            drawer.removeLine(edgeBF);
            drawer.removeLine(edgeCG);
            drawer.removeLine(edgeDH);
        }
    }
}
