﻿using System.Collections.Generic;
using Microsoft.Xna.Framework.Graphics;

namespace BEPUphysicsDrawer.Models
{
    /// <summary>
    /// Stores vertex and index data for the brute force renderer.
    /// </summary>
    internal class BruteDisplayObjectEntry
    {
        internal ModelDisplayObjectBase displayObject;
        internal BruteModelDrawer drawer;
        internal IndexBuffer indexBuffer;

        internal int[] indices;

        internal VertexBuffer vertexBuffer;
        internal VertexPositionNormalTexture[] vertices;

        internal BruteDisplayObjectEntry(BruteModelDrawer drawer, ModelDisplayObjectBase displayObject)
        {
            this.drawer = drawer;
            this.displayObject = displayObject;

            var tempVertices = new List<VertexPositionNormalTexture>();
            var tempIndices = new List<int>();
            displayObject.GetVertexData(tempVertices, tempIndices);
            vertices = new VertexPositionNormalTexture[tempVertices.Count];
            indices = new int[tempIndices.Count];
            tempVertices.CopyTo(vertices);
            tempIndices.CopyTo(indices);

            vertexBuffer = new VertexBuffer(displayObject.Drawer.Game.GraphicsDevice, vertices.Length * VertexPositionNormalTexture.SizeInBytes, BufferUsage.WriteOnly);
            indexBuffer = new IndexBuffer(displayObject.Drawer.Game.GraphicsDevice, indices.Length * 4, BufferUsage.WriteOnly, IndexElementSize.ThirtyTwoBits);

            vertexBuffer.SetData(vertices);
            indexBuffer.SetData(indices);
        }

        internal void Draw(Texture2D[] textures, GraphicsDevice device, BasicEffect effect)
        {
            effect.World = displayObject.WorldTransform;
            effect.Texture = textures[displayObject.TextureIndex];
            device.Vertices[0].SetSource(vertexBuffer, 0, VertexPositionNormalTexture.SizeInBytes);
            device.Indices = indexBuffer;
            effect.CommitChanges();

            device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, vertices.Length, 0, indices.Length / 3);
        }
    }
}